/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include <X11/X.h>
#include <X11/Xproto.h>
#include <X11/keysym.h>
#include "misc.h"
#include "input.h"
#include "../../mi/mipointer.h"

#include "commonOS.h"
#include "xosInput.h"

static void
xosChangePointerControl( pDev, pCtrl )
DevicePtr	pDev;
PtrCtrl		*pCtrl;
{
    /*
     * TBD (FOURTH STAGE)
     *    The DIX layer calls this function to change pointer acceleration.
     *    If your system supports pointer acceleration, add support here for
     * change the system's idea of the acceleration.  If you system doesn't
     * support pointer acceleration, you can implement it in software in 
     * ProcessPointerEvents.
     */
    return;
}

int
xosPointerProc(pPtr, what, argc, argv)
DevicePtr	pPtr;
int		what;
int		argc;
char		*argv[];
{
CARD8	map[_XOS_NUM_PTR_BTNS+1];
int	i;

    switch (what) {
	case DEVICE_INIT: 
	    {	
	    /* 
	     * TBD (THIRD STAGE) 
	     *    At this stage, we're only setting up the device. It shouldn't
             * report events yet.   If you need to probe for the existence of
	     * a pointer or initialize some driver or data structures, this is
	     * the place to do it.   You can store anything you want in
	     * pPtr->devicePrivate (of type pointer).
	     */
	    for (i=0;i<=_XOS_NUM_PTR_BTNS;i++)
		map[i] = i;
	    InitPointerDeviceStruct( pPtr, map, _XOS_NUM_PTR_BTNS, 
			miPointerGetMotionEvents, xosChangePointerControl, 
			miPointerGetMotionBufferSize());
	    }
	    break;
	case DEVICE_ON: 
	    /*
	     * TBD (THIRD STAGE)
	     *    The DIX layer is ready to get events from the device.  Open
	     * the device here or allow it to generate events.
	     */
            break;
	case DEVICE_OFF: 
            /*
	     * TBD (THIRD STAGE)
	     *    The DIX layer no longer wants events from the device.  
	     * Suspend or close it.
	     */
            break;
	case DEVICE_CLOSE:
            /*
	     * TBD (THIRD STAGE)
	     *    The DIX layer is about to free this device.   Close it down
	     * for good and free any private data structures.  If you stored
	     * anything in pPtr->devicePrivate, you should free it here.
	     */
	    break;
    }
    return Success;
}

