/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include <stdio.h>
#include "X.h"
#include "Xproto.h"

#include "misc.h"
#include "servermd.h"
#include "resource.h"
#include "window.h"
#include "scrnintstr.h"
#include "colormapst.h"

#include "ddxScreens.h"

#define _NEED_XXX_PRIVATE
#include "xxxDevInt.h"

	/*
	 * This file contains all of the colormap manipulation functions
	 * you must implement for a display with a single hardware colormap.
	 */

/***====================================================================***/

extern int      TellLostMap();
extern int      TellGainedMap();

/***====================================================================***/

static Bool
xxxGetInstalledColormap(pScreen,ppMap)
ScreenPtr	 pScreen;
ColormapPtr	*ppMap;
{
xxxScreenPrivPtr	pPriv;

    if (pScreen!=NULL) {
	pPriv= xxxPriv(pScreen);
	if ((pPriv!=NULL)&&(pPriv->pInstalledCMap)) {
	    *ppMap=	pPriv->pInstalledCMap;
	    return(TRUE);
	}
    }
    *ppMap=	NULL;
    return(FALSE);
}

/***====================================================================***/

static Bool
xxxSetInstalledColormap(pScreen,pNewMap)
ScreenPtr	pScreen;
ColormapPtr	pNewMap;
{
xxxScreenPrivPtr	pPriv;
ColormapPtr	pOldMap;

    if (pScreen!=NULL) {
	pPriv= xxxPriv(pScreen);
	if (pPriv!=NULL) {
	    pOldMap= pPriv->pInstalledCMap;
	    if (pOldMap!=NULL)
		WalkTree( pScreen, TellLostMap, &pOldMap->mid);
	    pPriv->pInstalledCMap=	pNewMap;
	    WalkTree( pScreen, TellGainedMap, &pNewMap->mid);
	    return(TRUE);
	}
    }
    return(FALSE);
}

/***====================================================================***/

void
xxxInstallColormap(pMap)
ColormapPtr	pMap;
{
Pixel		*pPixels;
xrgb		*pRGBDefs;
int      	entries = pMap->pVisual->ColormapEntries;
int         	 i;
ColormapPtr	pCurMap;

    if (xxxGetInstalledColormap(pMap->pScreen,&pCurMap)&&(pCurMap==pMap))
        return;

    pPixels=	(Pixel *)ALLOCATE_LOCAL( entries * sizeof(Pixel));
    pRGBDefs=	(xrgb *)ALLOCATE_LOCAL( entries * sizeof(xrgb));

    if ( pCurMap != NULL)
        WalkTree( pMap->pScreen, TellLostMap, &pCurMap->mid);
    xxxSetInstalledColormap(pMap->pScreen,pMap);

    for ( i=0; i<entries; i++) {
        pPixels[i] = i;
    }
    QueryColors( pMap, entries, pPixels, pRGBDefs);
    /* 
     * TBD (SECOND STAGE):
     * pRGBDefs contains the pixel values for the entire new colormap.
     * Do whatever os/device dependent things are necessary to update
     * the installed colormap.   If your hardware supports multiple
     * colormaps, this code has to be a lot more intelligent.
     */

    DEALLOCATE_LOCAL(pPixels);
    DEALLOCATE_LOCAL(pRGBDefs);
    return;
}

/***====================================================================***/

void
xxxUninstallColormap(pMap)
ColormapPtr	pMap;
{
ScreenPtr	pScreen= pMap->pScreen;
ColormapPtr	pCurMap;
ColormapPtr	pDfltMap;

    if ((!xxxGetInstalledColormap(pScreen,&pCurMap))||(pCurMap!=pMap))
	return;

    pDfltMap= (ColormapPtr)LookupIDByType( pScreen->defColormap,RT_COLORMAP);

    if (pMap==pDfltMap)
        return;

    (*pMap->pScreen->InstallColormap)(pDfltMap);
    return;
}

/***====================================================================***/

int
xxxListInstalledColormaps( pScreen, pMaps)
ScreenPtr	 pScreen;
Colormap	*pMaps;
{
ColormapPtr	pCurMap;

    if (xxxGetInstalledColormap(pScreen,&pCurMap)) {
	*pMaps= pCurMap->mid;
	return 1;
    }
    return 0;
}

/***====================================================================***/

void
xxxStoreColors(pMap, nDefs, pDefs)
ColormapPtr	 pMap;
int		 nDefs;
xColorItem	*pDefs;
{
ColormapPtr	pCurMap;

    /* 
     * Don't store colors unless this map is the active map
     * if your hardware supports multiple installed colormaps, this
     * check has to be lots smarter
     */
     
    if ((!xxxGetInstalledColormap(pMap->pScreen,&pCurMap))||(pCurMap!=pMap))
	return;

    /*
     * TBD (SECOND STAGE):
     * The pDefs parameter lists "nDefs" color values to be
     * changed.  Do any machine/os specific things necessary
     * to change those colormap entries here.
     */
    return;
}

