/*********************************************************************
Copyright 1992 by Digital Press/Digital Equipment Corporation, Elias
Israel, and Erik Fortune

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.  Digital Equipment Corporation and the
authors make no representations about the suitability of this software
for any purpose.  It is provided "as is" without express or implied
warranty.

DIGITAL EQUIPMENT CORPORATION AND THE AUTHORS DISCLAIM ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION OR THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

This material is reproduced with permission from _The X Window System
Server: X Version 11, Release 5_, by Elias Israel and Erik Fortune.
(Copyright 1992, Digital Press, Digital Equipment Corporation, One
Burlington Woods Drive, Burlington, MA 01803)
*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "os.h"

extern	void	CopyISOLatin1Lowered();

typedef struct _cdef {
	unsigned char	*name;
	unsigned short	 red;
	unsigned short	 green;
	unsigned short	 blue;
} ColorDefRec;


static	int		 nColors=	0;
static	int		 szColors=	0;
static	ColorDefRec	*colors=	NULL;

#define	RESIZE_INCREMENT	750
#define	MAX_NAME_LEN		512
#define	RGB_SUFFIX		".txt"

static int
OsAddColor(name,len,red,green,blue)
unsigned char	*name;
int	 	 len;
int	 	 red,green,blue;
{
    if (nColors>=szColors) {
	int	newSize;
	szColors+=	RESIZE_INCREMENT;
	newSize=	szColors*sizeof(ColorDefRec);
	if (colors==NULL)
	     colors=	(ColorDefRec *)xalloc(newSize);
	else colors=	(ColorDefRec *)xrealloc(colors,newSize);
	if (colors==NULL) {
	    ErrorF("Allocation failure reading colors database\n");
	    return(FALSE);
	}
    }
    colors[nColors].name=	(unsigned char *)xalloc(len+1);
    if (colors[nColors].name==NULL) {
	ErrorF("Allocation failure for color name\n");
	return(FALSE);
    }
    CopyISOLatin1Lowered(colors[nColors].name,name,len);
    colors[nColors].red=	(red*65535) / 255;
    colors[nColors].green=	(green*65535) / 255;
    colors[nColors].blue=	(blue*65535) / 255;
    nColors++;
    return(TRUE);
}

int
OsInitColors()
{
extern	 char *rgbPath;
char	 name[MAX_NAME_LEN];
FILE	*fil;
int	 items,lineNum;
int	 red,green,blue;
int	 ch,len;

    if (strlen(rgbPath)>(MAX_NAME_LEN-sizeof(RGB_SUFFIX)))
	return(FALSE);

    sprintf(name,"%s%s",rgbPath,RGB_SUFFIX);
    fil=	fopen(name,"r");
    if (fil==NULL) {
	ErrorF("Couldn't open colors database file %s\n",name);
	return(FALSE);
    }
    lineNum= 0;
    while ((items=fscanf(fil," %d %d %d ",&red,&green,&blue))==3) {
	while (((ch=getc(fil))!=EOF)&&(ch!='\n')&&(isspace(ch)))
	    /* conditional does the work */;
	len= 0;
	while ((ch!=EOF)&&(ch!='\n')) {
	    if (len<MAX_NAME_LEN)
		name[len++]=	ch;
	    ch=	getc(fil);
	}
	if (len>0) {
	    if (!OsAddColor(name,len,red,green,blue)) {
		fclose(fil);
		return(FALSE);
	    }
	}
	else {
	    ErrorF("No name on line %d of colors file %s%s\n",lineNum,
							rgbPath,RGB_SUFFIX);
	}
	lineNum++;
    }
    if ((items>0)||(ch=getc(fil))!=EOF) {
	fclose(fil);
	ErrorF("Warning!  Syntax error on line %d of colors file %s%s\n",
						lineNum,rgbPath,RGB_SUFFIX);
	return(FALSE);
    }
    fclose(fil);
    return(TRUE);
}

int
OsLookupColor( screen, name, len, pRed, pGreen, pBlue )
int		 screen;
char		*name;
int		 len;
unsigned short	*pRed;
unsigned short	*pGreen;
unsigned short	*pBlue;
{
register int	 i;
unsigned char	*lower;

    lower= (unsigned char *)ALLOCATE_LOCAL(len+1);
    if (lower==NULL)
	return(FALSE);
    CopyISOLatin1Lowered(lower,(unsigned char *)name,(int)len);

    for (i=0;i<nColors;i++) {
	if (strncmp(colors[i].name,lower,len)==0) {
	    if (pRed!=NULL)	*pRed=		colors[i].red;
	    if (pGreen!=NULL)	*pGreen=	colors[i].green;
	    if (pBlue!=NULL)	*pBlue=		colors[i].blue;
	    DEALLOCATE_LOCAL(lower);
	    return(TRUE);
	}
    }
    DEALLOCATE_LOCAL(lower);
    return FALSE;
}
